@@ -3,9 +3,9 @@ Crop - Crop a given size from one or multiple images.
 Crop is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,6 +33,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/fits.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/table.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/dimension.h>
 
 #include <gnuastro-internal/timing.h>
@@ -83,19 +84,6 @@ doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" will create cutouts, "
 
 
 
-/* Option groups particular to this program. */
-enum program_args_groups
-{
-  ARGS_GROUP_CENTER_GENERAL = GAL_OPTIONS_GROUP_AFTER_COMMON,
-  ARGS_GROUP_CENTER_SINGLE,
-  ARGS_GROUP_CENTER_CATALOG,
-  ARGS_GROUP_REGION,
-};
-
-
-
-
-
 
 
 
@@ -155,7 +143,7 @@ ui_initialize_options(struct cropparams *p,
 
         case GAL_OPTIONS_KEY_SEARCHIN:
         case GAL_OPTIONS_KEY_IGNORECASE:
-          cp->coptions[i].group=ARGS_GROUP_CENTER_CATALOG;
+          cp->coptions[i].group=UI_GROUP_CENTER_CATALOG;
           break;
         }
 
@@ -253,75 +241,6 @@ ui_parse_coordinate_mode(struct argp_option *option, char *arg,
 
 
 
-/* Parse the width and center coordinates from the comman-line or
-   configuration files. */
-void *
-ui_parse_width_and_center(struct argp_option *option, char *arg,
-                          char *filename, size_t lineno, void *junk)
-{
-  size_t i, nc;
-  double *darray;
-  gal_data_t *values;
-  char *str, sstr[GAL_OPTIONS_STATIC_MEM_FOR_VALUES];
-
-  /* We want to print the stored values. */
-  if(lineno==-1)
-    {
-      /* Set the value pointer to the correct type. */
-      values = *(gal_data_t **)(option->value);
-      darray = values->array;
-
-      /* Write the values into a string. */
-      nc=0;
-      for(i=0;i<values->size;++i)
-        {
-          if( nc > GAL_OPTIONS_STATIC_MEM_FOR_VALUES-100 )
-            error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s so we "
-                  "can address the problem. The number of necessary "
-                  "characters in the statically allocated string has become "
-                  "too close to %d", __func__, PACKAGE_BUGREPORT,
-                  GAL_OPTIONS_STATIC_MEM_FOR_VALUES);
-          nc += sprintf(sstr+nc, "%g,", darray[i]);
-        }
-      sstr[nc-1]='\0';
-
-      /* Copy the string into a dynamically allocated space, because it
-         will be freed later.*/
-      gal_checkset_allocate_copy(sstr, &str);
-      return str;
-    }
-  else
-    {
-      /* If the option is already set, then ignore it. */
-      if(option->set) return NULL;
-
-      /* Read the values. */
-      values=gal_options_parse_list_of_numbers(arg, filename, lineno);
-      *(gal_data_t **)(option->value) = values;
-
-      /* If we are on the width option, then make sure none of the values
-         are negative or zero. */
-      if(!strcmp(option->name, "width"))
-        {
-          darray=values->array;
-          for(i=0;i<values->size;++i)
-            if(darray[i]<=0.0f)
-              error_at_line(EXIT_FAILURE, 0, filename, lineno, "%g is <=0. "
-                            "The values to the `--width' option must be "
-                            "larger than zero. The complete input to this "
-                            "option was `%s' (%g is input number %zu)",
-                            darray[i], arg, darray[i], i+1);
-        }
-
-      /* The return value is only for printing mode. */
-      return NULL;
-    }
-}
-
-
-
-
-
 
 
 
@@ -345,7 +264,8 @@ ui_parse_width_and_center(struct argp_option *option, char *arg,
 static void
 ui_read_check_only_options(struct cropparams *p)
 {
-  int checksum;
+  double *darray;
+  int i, checksum;
 
   /* Make sure that only one of the crop definitions is given. */
   checksum = ( (p->center!=NULL)
@@ -375,11 +295,24 @@ ui_read_check_only_options(struct cropparams *p)
     }
 
 
+  /* The width values must not be negative. */
+  if(p->width)
+    {
+      darray=p->width->array;
+      for(i=0;i<p->width->size;++i)
+        if(darray[i]<=0.0f)
+          error(EXIT_FAILURE, 0, "%g is <=0. The values to the `--width' "
+                "option must be larger than zero. %g is input number %d to "
+                "this option", darray[i], darray[i], i+1);
+    }
+
+
   /* Section is currentlyl only defined in Image mode. */
   if(p->section && p->mode!=IMGCROP_MODE_IMG)
     error(EXIT_FAILURE, 0, "The `--section' option is only available in "
           "image coordinate mode, currently it doesn't work with WCS mode. "
-          "Please run with `--mode=img' and change the values accordingly");
+          "Please run with `--mode=img' and if necessary, change the "
+          "values accordingly");
 
 
   /* Sanity checks and mode setting based on the desired crop. */
@@ -667,7 +600,7 @@ ui_read_cols(struct cropparams *p)
 
   /* Read the desired columns from the file. */
   cols=gal_table_read(p->catname, p->cathdu, colstrs, p->cp.searchin,
-                      p->cp.ignorecase, p->cp.minmapsize);
+                      p->cp.ignorecase, p->cp.minmapsize, NULL);
 
 
   /* Set the number of objects (rows in each column). */
@@ -771,9 +704,9 @@ ui_prepare_center(struct cropparams *p)
       carray=p->center->array;
       for(i=0;i<ndim;++i)
         {
-          p->centercoords[i]=gal_data_malloc_array(GAL_TYPE_FLOAT64,
-                                                   1, __func__,
-                                                   "p->centercoords[i]");
+          p->centercoords[i]=gal_pointer_allocate(GAL_TYPE_FLOAT64,
+                                                  1, 0, __func__,
+                                                  "p->centercoords[i]");
           p->centercoords[i][0]=carray[i];
         }
     }
@@ -795,8 +728,9 @@ ui_make_log(struct cropparams *p)
   if(p->cp.log==0) return;
 
   /* Column to specify if the central pixels are filled. */
-  asprintf(&comment, "Are the central pixels filled? (1: yes, 0: no, "
-           "%u: not checked)", GAL_BLANK_UINT8);
+  if( asprintf(&comment, "Are the central pixels filled? (1: yes, 0: no, "
+               "%u: not checked)", GAL_BLANK_UINT8)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_list_data_add_alloc(&p->log, NULL, GAL_TYPE_UINT8, 1, &p->numout,
                           NULL, 1, p->cp.minmapsize, "CENTER_FILLED",
                           "bool", comment);
@@ -882,7 +816,7 @@ ui_preparations(struct cropparams *p)
           /* Set the basic information. */
           firsttype = p->type;
           firstndim = img->ndim;
-          p->bitnul = gal_blank_alloc_write(p->type);
+          p->bitnul = gal_fits_key_img_blank(p->type);
 
           /* Make sure the number of dimensions is supported. */
           if(firstndim>MAXDIM)
@@ -1021,13 +955,15 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct cropparams *p)
   if(!p->cp.quiet)
     {
       printf(PROGRAM_NAME" started on %s", ctime(&p->rawtime));
-      asprintf(&msg, "Read metadata of %zu dataset%s.", p->numin,
-              p->numin>1 ? "s" : "");
+      if( asprintf(&msg, "Read metadata of %zu dataset%s.", p->numin,
+                   p->numin>1 ? "s" : "")<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(&t1, msg, 1);
       if(p->numout>1)
         {
-          asprintf(&msg, "Will try making %zu crops (from catalog).",
-                   p->numout);
+          if( asprintf(&msg, "Will try making %zu crops (from catalog).",
+                       p->numout)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
           gal_timing_report(NULL, msg, 1);
         }
     }
