@@ -3,9 +3,9 @@ MakeNoise - Add noise to a dataset.
 MakeNoise is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -44,7 +44,8 @@ struct mknoiseparams
   /* From command-line */
   struct gal_options_common_params cp;   /* Common parameters.           */
   char        *inputname;    /* Input filename.                          */
-  double          stdadd;    /* Standard deviation constants.            */
+  double           sigma;    /* Total noise sigma (ignoring others).     */
+  double    instrumental;    /* Standard deviation constants.            */
   double       zeropoint;    /* Zeropoint magnitude of image.            */
   double  background_mag;    /* Background in magnitudes.                */
   uint8_t        envseed;    /* ==1, generate a random seed.             */
