@@ -3,9 +3,9 @@ MakeProfiles - Create mock astronomical profiles.
 MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,6 +33,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/git.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/threads.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/dimension.h>
 #include <gnuastro/statistics.h>
 
@@ -138,7 +139,8 @@ saveindividual(struct mkonthread *mkp)
     filename=p->mergedimgname;
   else
     {
-      asprintf(&filename, "%s%zu_%s", outdir, ibq->id, p->basename);
+      if( asprintf(&filename, "%s%zu_%s", outdir, ibq->id, p->basename)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_checkset_writable_remove(filename, 0, p->cp.dontdelete);
     }
 
@@ -151,7 +153,8 @@ saveindividual(struct mkonthread *mkp)
     {
       /* Allocate space for the corrected crpix and fill it in. Both
          `crpix' and `fpixel_i' are in FITS order. */
-      crpix=gal_data_malloc_array(GAL_TYPE_FLOAT64, ndim, __func__, "crpix");
+      crpix=gal_pointer_allocate(GAL_TYPE_FLOAT64, ndim, 0, __func__,
+                                 "crpix");
       for(i=0;i<ndim;++i)
         crpix[i] = ((double *)(p->crpix->array))[i] - os*(mkp->fpixel_i[i]-1);
 
@@ -166,7 +169,8 @@ saveindividual(struct mkonthread *mkp)
   /* Report if in verbose mode. */
   if(!p->cp.quiet)
     {
-      asprintf(&jobname, "%s created.", filename);
+      if( asprintf(&jobname, "%s created.", filename)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(NULL, jobname, 2);
       free(jobname);
     }
@@ -257,8 +261,7 @@ mkprof_build_single(struct mkonthread *mkp, long *fpixel_i, long *lpixel_i,
           /* If a crop is needed, set the starting pointer. */
           ind=gal_dimension_coord_to_index(ndim, ibq->image->dsize,
                                            start_indiv);
-          ptr=gal_data_ptr_increment(ibq->image->array, ind,
-                                     ibq->image->type);
+          ptr=gal_pointer_increment(ibq->image->array, ind, ibq->image->type);
         }
       else ptr=ibq->image->array;
       ibq->overlap_i=gal_data_alloc(ptr, ibq->image->type, ndim, dsize, NULL,
@@ -268,7 +271,7 @@ mkprof_build_single(struct mkonthread *mkp, long *fpixel_i, long *lpixel_i,
 
       /* Define the merged overlap tile. */
       ind=gal_dimension_coord_to_index(ndim, p->out->dsize, start_mrg);
-      ptr=gal_data_ptr_increment(p->out->array, ind, p->out->type);
+      ptr=gal_pointer_increment(p->out->array, ind, p->out->type);
       ibq->overlap_m=gal_data_alloc(ptr, p->out->type, ndim, dsize, NULL,
                                     0, -1, NULL, NULL, NULL);
       ibq->overlap_m->block=p->out;
@@ -537,8 +540,9 @@ mkprof_write(struct mkprofparams *p)
       ++complete;
       if(!p->cp.quiet && p->num>1)
         {
-          asprintf(&jobname, "row %zu complete, %zu left to go",
-                   ibq->id+1, num-complete);
+          if( asprintf(&jobname, "row %zu complete, %zu left to go",
+                       ibq->id+1, num-complete)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
           gal_timing_report(NULL, jobname, 2);
           free(jobname);
         }
@@ -577,7 +581,8 @@ mkprof_write(struct mkprofparams *p)
       /* In verbose mode, print the information. */
       if(!p->cp.quiet)
         {
-          asprintf(&jobname, "%s created.", p->mergedimgname);
+          if( asprintf(&jobname, "%s created.", p->mergedimgname)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
           gal_timing_report(&t1, jobname, 1);
           free(jobname);
         }
@@ -641,7 +646,7 @@ mkprof(struct mkprofparams *p)
      ignore it. */
   if(p->out)
     {
-      onaxes=gal_data_malloc_array(GAL_TYPE_LONG, ndim, __func__, "onaxes");
+      onaxes=gal_pointer_allocate(GAL_TYPE_LONG, ndim, 0, __func__, "onaxes");
       for(fi=0; fi < ndim; ++fi)
         {
           i=ndim-fi-1;
@@ -701,10 +706,12 @@ mkprof(struct mkprofparams *p)
   /* Write the log file. */
   if(p->cp.log)
     {
-      asprintf(&tmp, "Zeropoint: %g", p->zeropoint);
+      if( asprintf(&tmp, "Zeropoint: %g", p->zeropoint)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_list_str_add(&comments, tmp, 0);
+      gal_checkset_writable_remove(LOGFILENAME, 0, p->cp.dontdelete);
       gal_table_write_log(p->log, PROGRAM_STRING, &p->rawtime, comments,
-                          LOGFILENAME, p->cp.dontdelete, p->cp.quiet);
+                          LOGFILENAME, p->cp.quiet);
       gal_list_str_free(comments, 1);
     }
 
