@@ -3,9 +3,9 @@ Functions to work with FITS image data.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -39,6 +39,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/fits.h>
 #include <gnuastro/tile.h>
 #include <gnuastro/blank.h>
+#include <gnuastro/pointer.h>
 
 #include <gnuastro-internal/checkset.h>
 #include <gnuastro-internal/tableintern.h>
@@ -147,7 +148,10 @@ gal_fits_name_save_as_string(char *filename, char *hdu)
 {
   char *name;
   if( gal_fits_name_is_fits(filename) )
-    asprintf(&name, "%s (hdu: %s)", filename, hdu);
+    {
+      if( asprintf(&name, "%s (hdu: %s)", filename, hdu)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+    }
   else gal_checkset_allocate_copy(filename, &name);
   return name;
 }
@@ -671,7 +675,8 @@ gal_fits_hdu_open(char *filename, char *hdu, int iomode)
   fitsfile *fptr;
 
   /* Add hdu to filename: */
-  asprintf(&ffname, "%s[%s#]", filename, hdu);
+  if( asprintf(&ffname, "%s[%s#]", filename, hdu)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
 
   /* Open the FITS file: */
   if( fits_open_file(&fptr, ffname, iomode, &status) )
@@ -794,6 +799,63 @@ gal_fits_hdu_open_format(char *filename, char *hdu, int img0_tab1)
 /**************************************************************/
 /**********            Header keywords             ************/
 /**************************************************************/
+/* Return allocated pointer to the blank value to use in a FITS file header
+   keyword. */
+void *
+gal_fits_key_img_blank(uint8_t type)
+{
+  void *out=NULL, *tocopy=NULL;
+  uint8_t u8=0;          /* Equivalent of minimum in signed   with BZERO. */
+  int16_t s16=INT16_MAX; /* Equivalend of maximum in unsigned with BZERO. */
+  int32_t s32=INT32_MAX; /* Equivalend of maximum in unsigned with BZERO. */
+  int64_t s64=INT64_MAX; /* Equivalend of maximum in unsigned with BZERO. */
+
+  switch(type)
+    {
+    /* Types with no special treatment: */
+    case GAL_TYPE_BIT:
+    case GAL_TYPE_UINT8:
+    case GAL_TYPE_INT16:
+    case GAL_TYPE_INT32:
+    case GAL_TYPE_INT64:
+    case GAL_TYPE_FLOAT32:
+    case GAL_TYPE_FLOAT64:
+    case GAL_TYPE_COMPLEX32:
+    case GAL_TYPE_COMPLEX64:
+    case GAL_TYPE_STRING:
+    case GAL_TYPE_STRLL:
+      out = gal_blank_alloc_write(type);
+      break;
+
+    /* Types that need values from their opposite-signed types. */
+    case GAL_TYPE_INT8:      tocopy=&u8;      break;
+    case GAL_TYPE_UINT16:    tocopy=&s16;     break;
+    case GAL_TYPE_UINT32:    tocopy=&s32;     break;
+    case GAL_TYPE_UINT64:    tocopy=&s64;     break;
+
+    default:
+      error(EXIT_FAILURE, 0, "%s: type code %u not recognized as a Gnuastro "
+            "data type", __func__, type);
+    }
+
+  /* If `gal_blank_alloc_write' wasn't used (copy!=NULL), then allocate the
+     necessary space and fill it in. Note that the width of the signed and
+     unsigned values doesn't differ, so we can use the actual input
+     type. */
+  if(tocopy)
+    {
+      out = gal_pointer_allocate(type, 1, 0, __func__, "out");
+      memcpy(out, tocopy, gal_type_sizeof(type));
+    }
+
+  /* Return. */
+  return out;
+}
+
+
+
+
+
 /* CFITSIO doesn't remove the two single quotes around the string value, so
    the strings it reads are like: 'value ', or 'some_very_long_value'. To
    use the value, it is commonly necessary to remove the single quotes (and
@@ -894,8 +956,8 @@ gal_fits_key_read_from_ptr(fitsfile *fptr, gal_data_t *keysll,
            set the size and ndim to 1. But first allocate dsize if it
            wasn't already allocated. */
         if(tmp->dsize==NULL)
-          tmp->dsize=gal_data_malloc_array(GAL_TYPE_SIZE_T, 1, __func__,
-                                           "tmp->dsize");
+          tmp->dsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, 1, 0, __func__,
+                                          "tmp->dsize");
         tmp->ndim=tmp->size=tmp->dsize[0]=1;
 
         /* When the type is a string, `tmp->array' is an array of pointers
@@ -905,12 +967,12 @@ gal_fits_key_read_from_ptr(fitsfile *fptr, gal_data_t *keysll,
         switch(tmp->type)
           {
           case GAL_TYPE_STRING:
-            errno=0;
             tmp->array=strarray=( tmp->array
                                   ? tmp->array
-                                  : gal_data_malloc_array(tmp->type, 1,
-                                                          __func__,
-                                                          "tmp->array") );
+                                  : gal_pointer_allocate(tmp->type, 1, 0,
+                                                         __func__,
+                                                         "tmp->array") );
+            errno=0;
             valueptr=strarray[0]=malloc(FLEN_VALUE * sizeof *strarray[0]);
             if(strarray[0]==NULL)
               error(EXIT_FAILURE, errno, "%s: %zu bytes for strarray[0]",
@@ -920,16 +982,16 @@ gal_fits_key_read_from_ptr(fitsfile *fptr, gal_data_t *keysll,
           default:
             tmp->array=valueptr=( tmp->array
                                   ? tmp->array
-                                  : gal_data_malloc_array(tmp->type, 1,
-                                                          __func__,
-                                                          "tmp->array") );
+                                  : gal_pointer_allocate(tmp->type, 1, 0,
+                                                         __func__,
+                                                         "tmp->array") );
           }
 
         /* Allocate space for the keyword comment if necessary.*/
         if(readcomment)
           {
             errno=0;
-            tmp->comment=malloc(FLEN_COMMENT * sizeof *tmp->comment);
+            tmp->comment=calloc(FLEN_COMMENT, sizeof *tmp->comment);
             if(tmp->comment==NULL)
               error(EXIT_FAILURE, errno, "%s: %zu bytes for tmp->comment",
                     __func__, FLEN_COMMENT * sizeof *tmp->comment);
@@ -947,7 +1009,7 @@ gal_fits_key_read_from_ptr(fitsfile *fptr, gal_data_t *keysll,
           {
             /* Allocate space for the unit and read it in. */
             errno=0;
-            tmp->unit=malloc(FLEN_COMMENT * sizeof *tmp->unit);
+            tmp->unit=calloc(FLEN_COMMENT, sizeof *tmp->unit);
             if(tmp->unit==NULL)
               error(EXIT_FAILURE, errno, "%s: %zu bytes for tmp->unit",
                     __func__, FLEN_COMMENT * sizeof *tmp->unit);
@@ -959,7 +1021,7 @@ gal_fits_key_read_from_ptr(fitsfile *fptr, gal_data_t *keysll,
         else
           tmp->unit=NULL;
 
-        /* Read the keyword and place its value in the poitner. */
+        /* Read the keyword and place its value in the pointer. */
         fits_read_key(fptr, gal_fits_type_to_datatype(tmp->type),
                       tmp->name, valueptr, tmp->comment, &tmp->status);
 
@@ -969,7 +1031,6 @@ gal_fits_key_read_from_ptr(fitsfile *fptr, gal_data_t *keysll,
 
         /* Strings need to be cleaned (CFITSIO puts `'' around them with
            some (possiblly) extra space on the two ends of the string. */
-
       }
 }
 
@@ -1378,9 +1439,11 @@ gal_fits_img_info(fitsfile *fptr, int *type, size_t *ndim, size_t **dsize,
     gal_fits_io_error(status, NULL);
   *ndim=naxis;
 
+
   /* Convert bitpix to Gnuastro's known types. */
   *type=gal_fits_bitpix_to_type(bitpix);
 
+
   /* Define the names of the possibly existing important keywords about the
      dataset. We are defining these in the opposite order to be read by
      CFITSIO. The way Gnuastro writes the FITS keywords, the output will
@@ -1419,18 +1482,21 @@ gal_fits_img_info(fitsfile *fptr, int *type, size_t *ndim, size_t **dsize,
       ++i;
     }
 
+
+  /* If a type correction is necessary, then do it. */
   if( !isnan(bscale) || bzero_str )
     fits_type_correct(type, bscale, bzero_str);
 
 
   /* Allocate the array to keep the dimension size and fill it in, note
      that its order is the opposite of naxes. */
-  *dsize=gal_data_malloc_array(GAL_TYPE_INT64, *ndim, __func__, "dsize");
+  *dsize=gal_pointer_allocate(GAL_TYPE_INT64, *ndim, 0, __func__, "dsize");
   for(i=0; i<*ndim; ++i)
     (*dsize)[i]=naxes[*ndim-1-i];
 
 
-  /* Clean up. */
+  /* Clean up. Note that bzero_str, gets freed by `gal_data_free' (which is
+     called by `gal_list_data_free'. */
   gal_list_data_free(keysll);
 }
 
@@ -1477,7 +1543,7 @@ gal_fits_img_read(char *filename, char *hdu, size_t minmapsize)
      problem, the space will be written/read as 32-bit `long' any way,
      we'll just have a few empty bytes that will be freed anyway at the end
      of this function. */
-  fpixel=gal_data_malloc_array(GAL_TYPE_INT64, ndim, __func__, "fpixel");
+  fpixel=gal_pointer_allocate(GAL_TYPE_INT64, ndim, 0, __func__, "fpixel");
   for(i=0;i<ndim;++i) fpixel[i]=1;
 
 
@@ -1485,6 +1551,8 @@ gal_fits_img_read(char *filename, char *hdu, size_t minmapsize)
   img=gal_data_alloc(NULL, type, ndim, dsize, NULL, 0, minmapsize,
                      name, unit, NULL);
   blank=gal_blank_alloc_write(type);
+  if(name) free(name);
+  if(unit) free(unit);
   free(dsize);
 
 
@@ -1546,9 +1614,10 @@ gal_fits_img_read_kernel(char *filename, char *hdu, size_t minmapsize)
   gal_data_t *kernel;
   float *f, *fp, tmp;
 
-  /* Read the image as a float */
+  /* Read the image as a float and if it has a WCS structure, free it. */
   kernel=gal_fits_img_read_to_type(filename, hdu, GAL_TYPE_FLOAT32,
                                    minmapsize);
+  if(kernel->wcs) { wcsfree(kernel->wcs); kernel->wcs=NULL; }
 
   /* Check if the size along each dimension of the kernel is an odd
      number. If they are all an odd number, then the for each dimension,
@@ -1601,9 +1670,9 @@ gal_fits_img_write_to_ptr(gal_data_t *input, char *filename)
   fitsfile *fptr;
   uint64_t *u64, *u64f;
   long fpixel=1, *naxes;
+  char *wcsstr, *u64key;
   size_t i, ndim=input->ndim;
   int nkeyrec, hasblank, status=0, datatype=0;
-  char *wcsstr, *u64key;
   gal_data_t *i64data, *towrite, *block=gal_tile_block(input);
 
   /* If the input is a tile (isn't a contiguous region of memory), then
@@ -1612,9 +1681,10 @@ gal_fits_img_write_to_ptr(gal_data_t *input, char *filename)
   hasblank=gal_blank_present(towrite, 0);
 
   /* Allocate the naxis area. */
-  naxes=gal_data_malloc_array( ( sizeof(long)==8
-                                 ? GAL_TYPE_INT64
-                                 : GAL_TYPE_INT32 ), ndim, __func__, "naxes");
+  naxes=gal_pointer_allocate( ( sizeof(long)==8
+                                ? GAL_TYPE_INT64
+                                : GAL_TYPE_INT32 ), ndim, 0, __func__,
+                              "naxes");
 
 
   /* Open the file for writing */
@@ -1710,7 +1780,7 @@ gal_fits_img_write_to_ptr(gal_data_t *input, char *filename)
         break;
 
       default:
-        blank=gal_blank_alloc_write(towrite->type);
+        blank=gal_fits_key_img_blank(towrite->type);
         if(fits_write_key(fptr, datatype, "BLANK", blank,
                           "Pixels with no data.", &status) )
           gal_fits_io_error(status, "adding the BLANK keyword");
@@ -2275,6 +2345,65 @@ gal_fits_tab_info(char *filename, char *hdu, size_t *numcols,
 
 
 
+/* Read CFITSIO un-readable (INF, -INF or NAN) floating point values in
+   FITS ASCII tables. */
+static void
+fits_tab_read_ascii_float_special(char *filename, char *hdu, fitsfile *fptr,
+                                  gal_data_t *out, size_t colnum,
+                                  size_t numrows, size_t minmapsize)
+{
+  double tmp;
+  char **strarr;
+  char *tailptr;
+  gal_data_t *strrows;
+  size_t i, colwidth=50;
+  int anynul=0, status=0;
+
+  /* Allocate the dataset to keep the string values. */
+  strrows=gal_data_alloc(NULL, GAL_TYPE_STRING, 1, &numrows, NULL, 0,
+                         minmapsize, NULL, NULL, NULL);
+
+  /* Allocate the space to keep the string values. */
+  strarr=strrows->array;
+  for(i=0;i<numrows;++i)
+    {
+      errno=0;
+      strarr[i]=calloc(colwidth, sizeof *strarr[i]);
+      if(strarr[i]==NULL)
+        error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for "
+              "strarr[%zu]", __func__, colwidth * sizeof *strarr[i], i);
+    }
+
+  /* Read the column as a string. */
+  fits_read_col(fptr, TSTRING, colnum, 1, 1, out->size, NULL, strrows->array,
+                &anynul, &status);
+  gal_fits_io_error(status, NULL);
+
+  /* Convert the strings to float. */
+  for(i=0;i<numrows;++i)
+    {
+      /* Parse the string. */
+      tmp=strtod(strarr[i], &tailptr);
+      if(tailptr==strarr[i])
+        error(EXIT_FAILURE, 0, "%s (hdu %s): couldn't parse row %zu of "
+              "column %zu (with value `%s') as a floating point number",
+              filename, hdu, i+1, colnum, strarr[i]);
+
+      /* Write it into the output dataset. */
+      if(out->type==GAL_TYPE_FLOAT32)
+        ((float *)(out->array))[i]=tmp;
+      else
+        ((double *)(out->array))[i]=tmp;
+    }
+
+  /* Clean up. */
+  gal_data_free(strrows);
+}
+
+
+
+
+
 /* Read the column indexs given in the `indexll' linked list from a FITS
    table into a linked list of data structures, note that this is a
    low-level function, so the output data linked list is the inverse of the
@@ -2282,7 +2411,7 @@ gal_fits_tab_info(char *filename, char *hdu, size_t *numcols,
 gal_data_t *
 gal_fits_tab_read(char *filename, char *hdu, size_t numrows,
                   gal_data_t *allcols, gal_list_sizet_t *indexll,
-                  int minmapsize)
+                  size_t minmapsize)
 {
   size_t i=0;
   void *blank;
@@ -2290,12 +2419,17 @@ gal_fits_tab_read(char *filename, char *hdu, size_t numrows,
   char **strarr;
   fitsfile *fptr;
   gal_data_t *out=NULL;
-  int status=0, anynul=0;
   gal_list_sizet_t *ind;
+  int isfloat, status=0, anynul=0, hdutype;
 
   /* Open the FITS file */
   fptr=gal_fits_hdu_open_format(filename, hdu, 1);
 
+  /* See if its a Binary or ASCII table (necessary for floating point blank
+     values). */
+  if (fits_get_hdu_type(fptr, &hdutype, &status) )
+    gal_fits_io_error(status, NULL);
+
   /* Pop each index and read/store the array. */
   for(ind=indexll; ind!=NULL; ind=ind->next)
     {
@@ -2311,24 +2445,53 @@ gal_fits_tab_read(char *filename, char *hdu, size_t numrows,
          disp_width element of the data structure, which is done
          automatically in `gal_fits_table_info'. */
       if(out->type==GAL_TYPE_STRING)
-        for(i=0;i<numrows;++i)
-          {
-            strarr=out->array;
-            errno=0;
-            strarr[i]=calloc(allcols[ind->v].disp_width+1, sizeof *strarr[i]);
-            if(strarr[i]==NULL)
-              error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for "
-                    "strarr[%zu]", __func__,
-                    (allcols[ind->v].disp_width+1) * sizeof *strarr[i], i);
-          }
+        {
+          strarr=out->array;
+          for(i=0;i<numrows;++i)
+            {
+              errno=0;
+              strarr[i]=calloc(allcols[ind->v].disp_width+1,
+                               sizeof *strarr[i]);
+              if(strarr[i]==NULL)
+                error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for "
+                      "strarr[%zu]", __func__,
+                      (allcols[ind->v].disp_width+1) * sizeof *strarr[i], i);
+            }
+        }
 
       /* Allocate a blank value for the given type and read/store the
-         column using CFITSIO. Afterwards, free the blank value. */
-      blank=gal_blank_alloc_write(out->type);
+         column using CFITSIO. Note that for binary tables, we only need
+         blank values for integer types. For binary floating point types,
+         the FITS standard defines blanks as NaN (same as almost any other
+         software like Gnuastro). However if a blank value is specified,
+         CFITSIO will convert other special numbers like `inf' to NaN
+         also. We want to be able to distringuish `inf' and NaN here, so
+         for floating point types in binary tables, we won't define any
+         blank value. In ASCII tables, CFITSIO doesn't read the `NAN'
+         values (that it has written itself) unless we specify a blank
+         pointer/value. */
+      isfloat = out->type==GAL_TYPE_FLOAT32 || out->type==GAL_TYPE_FLOAT64;
+      blank = ( ( hdutype==BINARY_TBL && isfloat )
+                ? NULL
+                : gal_blank_alloc_write(out->type) );
       fits_read_col(fptr, gal_fits_type_to_datatype(out->type), ind->v+1,
                     1, 1, out->size, blank, out->array, &anynul, &status);
+
+      /* In the ASCII table format, CFITSIO might not be able to read `INF'
+         or `-INF'. In this case, it will set status to `BAD_C2D' or
+         `BAD_C2F'. So, we'll use our own parser for the column values. */
+      if( hdutype==ASCII_TBL
+          && isfloat
+          && (status==BAD_C2D || status==BAD_C2F) )
+        {
+          fits_tab_read_ascii_float_special(filename, hdu, fptr, out,
+                                            ind->v+1, numrows, minmapsize);
+          status=0;
+        }
+
+      /* Clean up and sanity check. */
+      if(blank) free(blank);
       gal_fits_io_error(status, NULL);
-      free(blank);
     }
 
   /* Close the FITS file */
@@ -2425,8 +2588,10 @@ fits_table_prepare_arrays(gal_data_t *cols, size_t numcols, int tableformat,
   for(col=cols; col!=NULL; col=col->next)
     {
       /* Set the `ttype' and `tunit' values: */
-      asprintf(&ttype[i], "%s", col->name ? col->name : "");
-      asprintf(&tunit[i], "%s", col->unit ? col->unit : "");
+      if( asprintf(&ttype[i], "%s", col->name ? col->name : "")<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+      if( asprintf(&tunit[i], "%s", col->unit ? col->unit : "")<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
 
 
       /* FITS's TFORM depends on the type of FITS table, so work
@@ -2468,13 +2633,15 @@ fits_table_prepare_arrays(gal_data_t *cols, size_t numcols, int tableformat,
               case GAL_TYPE_INT32:
               case GAL_TYPE_UINT64:
               case GAL_TYPE_INT64:
-                asprintf(&tform[i], "%c%d", fmt[0], col->disp_width);
+                if( asprintf(&tform[i], "%c%d", fmt[0], col->disp_width)<0 )
+                  error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
                 break;
 
               case GAL_TYPE_FLOAT32:
               case GAL_TYPE_FLOAT64:
-                asprintf(&tform[i], "%c%d.%d", fmt[0], col->disp_width,
-                         col->disp_precision);
+                if( asprintf(&tform[i], "%c%d.%d", fmt[0], col->disp_width,
+                             col->disp_precision)<0 )
+                  error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
                 break;
 
               default:
@@ -2492,9 +2659,15 @@ fits_table_prepare_arrays(gal_data_t *cols, size_t numcols, int tableformat,
           col->disp_width=fits_string_fixed_alloc_size(col);
           fmt[0]=gal_fits_type_to_bin_tform(col->type);
           if( col->type==GAL_TYPE_STRING )
-            asprintf(&tform[i], "%d%c", col->disp_width, fmt[0]);
+            {
+              if( asprintf(&tform[i], "%d%c", col->disp_width, fmt[0])<0 )
+                error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+            }
           else
-            asprintf(&tform[i], "%c", fmt[0]);
+            {
+              if( asprintf(&tform[i], "%c", fmt[0])<0 )
+                error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+            }
           break;
 
         default:
@@ -2530,7 +2703,8 @@ fits_write_tnull_tcomm(fitsfile *fptr, gal_data_t *col, int tableformat,
     case GAL_TABLE_FORMAT_AFITS:
 
       /* Print the keyword and value. */
-      asprintf(&keyname, "TNULL%zu", colnum);
+      if( asprintf(&keyname, "TNULL%zu", colnum)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       blank=gal_blank_as_string(col->type, col->disp_width);
 
       /* When in exponential form (`tform' starting with `E'), CFITSIO
@@ -2558,7 +2732,8 @@ fits_write_tnull_tcomm(fitsfile *fptr, gal_data_t *col, int tableformat,
           && col->type!=GAL_TYPE_STRING )
         {
           blank=gal_blank_alloc_write(col->type);
-          asprintf(&keyname, "TNULL%zu", colnum);
+          if( asprintf(&keyname, "TNULL%zu", colnum)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
           fits_write_key(fptr, gal_fits_type_to_datatype(col->type),
                          keyname, blank, "blank value for this column",
                          &status);
@@ -2576,8 +2751,10 @@ fits_write_tnull_tcomm(fitsfile *fptr, gal_data_t *col, int tableformat,
   /* Write the comments if there is any. */
   if(col->comment)
     {
-      asprintf(&keyname, "TCOMM%zu", colnum);
-      asprintf(&bcomment, "comment for field %zu", colnum);
+      if( asprintf(&keyname, "TCOMM%zu", colnum)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+      if( asprintf(&bcomment, "comment for field %zu", colnum)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       fits_write_key(fptr, TSTRING, keyname, col->comment,
                      bcomment, &status);
       gal_fits_io_error(status, NULL);
@@ -2594,7 +2771,7 @@ fits_write_tnull_tcomm(fitsfile *fptr, gal_data_t *col, int tableformat,
    table.*/
 void
 gal_fits_tab_write(gal_data_t *cols, gal_list_str_t *comments,
-                   int tableformat, char *filename, int dontdelete)
+                   int tableformat, char *filename, char *extname)
 {
   void *blank;
   fitsfile *fptr;
@@ -2616,16 +2793,11 @@ gal_fits_tab_write(gal_data_t *cols, gal_list_str_t *comments,
     }
 
 
-  /* Remove the output if it already exists. */
-  gal_checkset_writable_remove(filename, 0, dontdelete);
+  /* Open the FITS file for writing. */
+  fptr=gal_fits_open_to_write(filename);
 
 
-  /* Create the FITS file */
-  fits_create_file(&fptr, filename, &status);
-  gal_fits_io_error(status, NULL);
-
-
-  /* prepare necessary arrays and if integer type columns have blank
+  /* Prepare necessary arrays and if integer type columns have blank
      values, write the TNULLn keywords into the FITS file. */
   fits_table_prepare_arrays(cols, numcols, tableformat,
                             &tform, &ttype, &tunit);
@@ -2635,7 +2807,7 @@ gal_fits_tab_write(gal_data_t *cols, gal_list_str_t *comments,
      `fits_table_prepare_arrays'. */
   tbltype = tableformat==GAL_TABLE_FORMAT_AFITS ? ASCII_TBL : BINARY_TBL;
   fits_create_tbl(fptr, tbltype, numrows, numcols, ttype, tform, tunit,
-                  "table", &status);
+                  extname, &status);
   gal_fits_io_error(status, NULL);
 
 
