@@ -3,9 +3,9 @@ Interpolate - Fill blank values in a dataset
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,6 +31,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/list.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/blank.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/threads.h>
 #include <gnuastro/dimension.h>
 #include <gnuastro/statistics.h>
@@ -44,6 +45,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /*********************************************************************/
 /********************      Nearest neighbor       ********************/
+/***************         (Dimension agnostic)         ****************/
 /*********************************************************************/
 /* These are bit-flags, so we're using hexadecimal notation. */
 #define INTERPOLATE_FLAGS_NO       0
@@ -92,12 +94,12 @@ interpolate_close_neighbors_on_thread(void *in_prm)
   size_t ngb_counter, dist, pind, *dinc;
   size_t i, index, fullind, chstart=0, ndim=input->ndim;
   gal_data_t *median, *tin, *tout, *tnear, *nearest=NULL;
-  size_t *icoord=gal_data_malloc_array(GAL_TYPE_SIZE_T, ndim, __func__,
-                                       "icoord");
-  size_t *ncoord=gal_data_malloc_array(GAL_TYPE_SIZE_T, ndim, __func__,
-                                       "ncoord");
   size_t size = (correct_index ? tl->tottilesinch : input->size);
   size_t *dsize = (correct_index ? tl->numtilesinch : input->dsize);
+  size_t *icoord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
+                                      "icoord");
+  size_t *ncoord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
+                                      "ncoord");
   uint8_t *fullflag=&prm->thread_flags[tprm->id*input->size], *flag=fullflag;
 
 
@@ -113,8 +115,8 @@ interpolate_close_neighbors_on_thread(void *in_prm)
   tin=input;
   for(tvll=prm->ngb_vals; tvll!=NULL; tvll=tvll->next)
     {
-      nv=gal_data_ptr_increment(tvll->v, tprm->id*prm->numneighbors,
-                                input->type);
+      nv=gal_pointer_increment(tvll->v, tprm->id*prm->numneighbors,
+                               input->type);
       gal_list_data_add_alloc(&nearest, nv, tin->type, 1, &prm->numneighbors,
                               NULL, 0, -1, NULL, NULL, NULL);
       tin=tin->next;
@@ -138,8 +140,8 @@ interpolate_close_neighbors_on_thread(void *in_prm)
           tin=input;
           for(tout=prm->out; tout!=NULL; tout=tout->next)
             {
-              memcpy(gal_data_ptr_increment(tout->array, fullind, tin->type),
-                     gal_data_ptr_increment(tin->array,  fullind, tin->type),
+              memcpy(gal_pointer_increment(tout->array, fullind, tin->type),
+                     gal_pointer_increment(tin->array,  fullind, tin->type),
                      gal_type_sizeof(tin->type));
               tin=tin->next;
             }
@@ -163,7 +165,7 @@ interpolate_close_neighbors_on_thread(void *in_prm)
           chstart = (fullind / tl->tottilesinch) * tl->tottilesinch;
 
           /* Set the channel's starting pointer for the flags. */
-          flag = gal_data_ptr_increment(fullflag, chstart, GAL_TYPE_UINT8);
+          flag = gal_pointer_increment(fullflag, chstart, GAL_TYPE_UINT8);
         }
       else
         {
@@ -200,10 +202,10 @@ interpolate_close_neighbors_on_thread(void *in_prm)
               tin=input;
               for(tnear=nearest; tnear!=NULL; tnear=tnear->next)
                 {
-                  memcpy(gal_data_ptr_increment(tnear->array, ngb_counter,
-                                                tin->type),
-                         gal_data_ptr_increment(tin->array, chstart+pind,
-                                                tin->type),
+                  memcpy(gal_pointer_increment(tnear->array, ngb_counter,
+                                               tin->type),
+                         gal_pointer_increment(tin->array, chstart+pind,
+                                               tin->type),
                          gal_type_sizeof(tin->type));
                   tin=tin->next;
                 }
@@ -257,7 +259,7 @@ interpolate_close_neighbors_on_thread(void *in_prm)
         {
           /* Find the median and copy it. */
           median=gal_statistics_median(tnear, 1);
-          memcpy(gal_data_ptr_increment(tout->array, fullind, tout->type),
+          memcpy(gal_pointer_increment(tout->array, fullind, tout->type),
                  median->array, gal_type_sizeof(tout->type));
 
           /* Clean up and go to next array. */
@@ -333,8 +335,8 @@ gal_interpolate_close_neighbors(gal_data_t *input,
   int permute=(tl && tl->totchannels>1 && tl->workoverch);
 
 
-  /* If there are no blank values in the array we should only fill blank
-     values, then simply copy the input and abort. */
+  /* If there are no blank values in the array, AND we should only fill
+     blank values, then simply copy the input and abort. */
   if( (input->flag | GAL_DATA_FLAG_BLANK_CH)     /* Zero bit is meaningful.*/
       && !(input->flag | GAL_DATA_FLAG_HASBLANK) /* There are no blanks.   */
       && onlyblank )                             /* Only interpolate blank.*/
@@ -377,8 +379,8 @@ gal_interpolate_close_neighbors(gal_data_t *input,
                          input->wcs, 0, input->minmapsize, NULL,
                          input->unit, NULL);
   gal_list_void_add(&prm.ngb_vals,
-                    gal_data_malloc_array(input->type, ngbvnum, __func__,
-                                          "prm.ngb_vals"));
+                    gal_pointer_allocate(input->type, ngbvnum, 0, __func__,
+                                         "prm.ngb_vals"));
 
 
   /* If we are given a list of datasets, make the necessary
@@ -390,7 +392,7 @@ gal_interpolate_close_neighbors(gal_data_t *input,
     for(tin=input->next; tin!=NULL; tin=tin->next)
       {
         /* A small sanity check. */
-        if( gal_data_dsize_is_different(input, tin) )
+        if( gal_dimension_is_different(input, tin) )
           error(EXIT_FAILURE, 0, "%s: all datasets in the list must have "
                 "the same dimension and size", __func__);
 
@@ -401,8 +403,8 @@ gal_interpolate_close_neighbors(gal_data_t *input,
 
         /* Allocate the space for the neighbor values of this input. */
         gal_list_void_add(&prm.ngb_vals,
-                          gal_data_malloc_array(tin->type, ngbvnum, __func__,
-                                                "prm.ngb_vals"));
+                          gal_pointer_allocate(tin->type, ngbvnum, 0,
+                                               __func__, "prm.ngb_vals"));
       }
   gal_list_data_reverse(&prm.out);
   gal_list_void_reverse(&prm.ngb_vals);
@@ -410,9 +412,9 @@ gal_interpolate_close_neighbors(gal_data_t *input,
 
   /* Allocate space for all the flag values of all the threads here (memory
      in each thread is limited) and this is cleaner. */
-  prm.thread_flags=gal_data_malloc_array(GAL_TYPE_UINT8,
-                                         numthreads*input->size, __func__,
-                                         "prm.thread_flags");
+  prm.thread_flags=gal_pointer_allocate(GAL_TYPE_UINT8,
+                                        numthreads*input->size, 0, __func__,
+                                        "prm.thread_flags");
 
 
   /* Spin off the threads. */
@@ -446,3 +448,291 @@ gal_interpolate_close_neighbors(gal_data_t *input,
   gal_list_void_free(prm.ngb_vals, 1);
   return prm.out;
 }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+/*********************************************************************/
+/********************          1D on grid         ********************/
+/*********************************************************************/
+gsl_spline *
+gal_interpolate_1d_make_gsl_spline(gal_data_t *X, gal_data_t *Y, int type_1d)
+{
+  size_t i, c;
+  double *x, *y;
+  gal_data_t *Xd, *Yd;
+  gsl_spline *spline=NULL;
+  const gsl_interp_type *itype=NULL;
+  int Yhasblank=gal_blank_present(Y, 0);
+
+  /* A small sanity check. */
+  if(Y->ndim!=1)
+    error(EXIT_FAILURE, 0, "%s: input dataset is not 1D (it is %zuD)",
+          __func__, Y->ndim);
+  if(X)
+    {
+      if( gal_dimension_is_different(X, Y) )
+        error(EXIT_FAILURE, 0, "%s: when two inputs are given, they must "
+              "have the same dimensions. X has %zu elements, while Y has "
+              "%zu", __func__, X->size, Y->size);
+      if(gal_blank_present(X, 0))
+        error(EXIT_FAILURE, 0, "%s: the X dataset has blank elements",
+              __func__);
+    }
+
+  /* Set the interpolation type. */
+  switch(type_1d)
+    {
+    case GAL_INTERPOLATE_1D_LINEAR:
+      itype=gsl_interp_linear;           break;
+    case GAL_INTERPOLATE_1D_POLYNOMIAL:
+      itype=gsl_interp_polynomial;       break;
+    case GAL_INTERPOLATE_1D_CSPLINE:
+      itype=gsl_interp_cspline;          break;
+    case GAL_INTERPOLATE_1D_CSPLINE_PERIODIC:
+      itype=gsl_interp_cspline_periodic; break;
+    case GAL_INTERPOLATE_1D_AKIMA:
+      itype=gsl_interp_akima;            break;
+    case GAL_INTERPOLATE_1D_AKIMA_PERIODIC:
+      itype=gsl_interp_akima_periodic;   break;
+    case GAL_INTERPOLATE_1D_STEFFEN:
+      itype=gsl_interp_steffen;          break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: code %d not recognizable for the GSL "
+            "interpolation type", __func__, type_1d);
+    }
+
+  /* Initializations. Note that if Y doesn't have any blank elements and is
+     already in `double' type, then we don't need to make a copy. */
+  Yd = ( (Yhasblank || Y->type!=GAL_TYPE_FLOAT64)
+         ? gal_data_copy_to_new_type(Y, GAL_TYPE_FLOAT64)
+         : Y );
+  Xd = ( X
+         /* Has to be `Yhasblank', we KNOW X doesn't have blank values. */
+         ? ( (Yhasblank || X->type!=GAL_TYPE_FLOAT64)
+             ? gal_data_copy_to_new_type(X, GAL_TYPE_FLOAT64)
+             : X )
+         : gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, Y->dsize, NULL,
+                          0, -1, NULL, NULL, NULL) );
+
+  /* Fill in the X axis values while also removing NaN/blank elements. */
+  c=0;
+  x=Xd->array;
+  y=Yd->array;
+  for(i=0;i<Yd->size;++i)
+    if( !isnan(y[i]) )
+      {
+        y[ c   ] = y[i];
+	x[ c++ ] = X ? x[i] : i;
+      }
+
+  /* Make sure we have enough valid points for interpolation. */
+  if( c>=gsl_interp_type_min_size(itype) )
+    {
+      spline=gsl_spline_alloc(itype, c);
+      gsl_spline_init(spline, x, y, c);
+    }
+  else
+    spline=NULL;
+
+  /* Clean up and return. */
+  if(Xd!=X) gal_data_free(Xd);
+  if(Yd!=Y) gal_data_free(Yd);
+  return spline;
+}
+
+
+
+
+
+/* Return 0 if all blanks were filled. */
+static int
+interpolate_1d_blank_write(gal_data_t *in, gsl_spline *spline,
+			   gsl_interp_accel *acc)
+{
+  double tmp;
+  int hasblank=0;
+  uint8_t  *su8 =in->array, *u8 =in->array, *u8f =u8 +in->size;
+  int8_t   *si8 =in->array, *i8 =in->array, *i8f =i8 +in->size;
+  uint16_t *su16=in->array, *u16=in->array, *u16f=u16+in->size;
+  int16_t  *si16=in->array, *i16=in->array, *i16f=i16+in->size;
+  uint32_t *su32=in->array, *u32=in->array, *u32f=u32+in->size;
+  int32_t  *si32=in->array, *i32=in->array, *i32f=i32+in->size;
+  uint64_t *su64=in->array, *u64=in->array, *u64f=u64+in->size;
+  int64_t  *si64=in->array, *i64=in->array, *i64f=i64+in->size;
+  float    *sf32=in->array, *f32=in->array, *f32f=f32+in->size;
+  double   *sf64=in->array, *f64=in->array, *f64f=f64+in->size;
+
+  switch(in->type)
+    {
+    case GAL_TYPE_UINT8:
+      do
+        if(*u8==GAL_BLANK_UINT8)
+          {
+            /* If the evaluation is good, this function will return 0. */
+            if( gsl_spline_eval_e(spline, u8-su8, acc, &tmp)==0 )
+              *u8=tmp;
+            else hasblank=1;
+          }
+      while(++u8<u8f);
+      break;
+    case GAL_TYPE_INT8:
+      do
+        if(*i8==GAL_BLANK_INT8)
+          {
+            if( gsl_spline_eval_e(spline, i8-si8, acc, &tmp)==0 )
+              *u16=tmp;
+            else hasblank=1;
+          }
+      while(++i8<i8f);
+      break;
+    case GAL_TYPE_UINT16:
+      do
+        if(*u16==GAL_BLANK_UINT16)
+          {
+            if( gsl_spline_eval_e(spline, u16-su16, acc, &tmp)==0 )
+              *u16=tmp;
+            else hasblank=1;
+          }
+      while(++u16<u16f);
+      break;
+    case GAL_TYPE_INT16:
+      do
+        if(*i16==GAL_BLANK_INT16)
+          {
+            if( gsl_spline_eval_e(spline, i16-si16, acc, &tmp)==0 )
+              *i16=tmp;
+            else hasblank=1;
+          }
+      while(++i16<i16f);
+      break;
+    case GAL_TYPE_UINT32:
+      do
+        if(*u32==GAL_BLANK_UINT32)
+          {
+            if( gsl_spline_eval_e(spline, u32-su32, acc, &tmp)==0 )
+              *u32=tmp;
+            else hasblank=1;
+          }
+      while(++u32<u32f);
+      break;
+    case GAL_TYPE_INT32:
+      do
+        if(*i32==GAL_BLANK_INT32)
+          {
+            if( gsl_spline_eval_e(spline, i32-si32, acc, &tmp)==0 )
+              *i32=tmp;
+            else hasblank=1;
+          }
+      while(++i32<i32f);
+      break;
+    case GAL_TYPE_UINT64:
+      do
+        if(*u64==GAL_BLANK_UINT64)
+          {
+            if( gsl_spline_eval_e(spline, u64-su64, acc, &tmp)==0 )
+              *u64=tmp;
+            else hasblank=1;
+          }
+      while(++u64<u64f);
+      break;
+    case GAL_TYPE_INT64:
+      do
+        if(*i64==GAL_BLANK_INT64)
+          {
+            if( gsl_spline_eval_e(spline, i64-si64, acc, &tmp)==0 )
+              *i64=tmp;
+            else hasblank=1;
+          }
+      while(++i64<i64f);
+      break;
+    case GAL_TYPE_FLOAT32:
+      do
+        if(isnan(*f32))
+          {
+            if( gsl_spline_eval_e(spline, f32-sf32, acc, &tmp)==0 )
+              *f32=tmp;
+            else hasblank=1;
+          }
+      while(++f32<f32f);
+      break;
+    case GAL_TYPE_FLOAT64:
+      do
+        if(isnan(*f64))
+          {
+            if( gsl_spline_eval_e(spline, f64-sf64, acc, f64) )
+              hasblank=1;
+          }
+      while(++f64<f64f);
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: code %d is not a recognized data type",
+	    __func__, in->type);
+    }
+  return hasblank;
+}
+
+
+
+
+
+void
+gal_interpolate_1d_blank(gal_data_t *in, int type_1d)
+{
+  int hasblank;
+  gsl_spline *spline;
+  gsl_interp_accel *acc;
+
+  /* If there are no blank elements, just return. */
+  if(!gal_blank_present(in, 1)) return;
+
+  /* Initialize the necessary structures. */
+  spline=gal_interpolate_1d_make_gsl_spline(NULL, in, type_1d);
+
+  /* If any interpolation structure was actually made. */
+  if(spline)
+    {
+      /* Write the values in the blank elements. */
+      acc=gsl_interp_accel_alloc();
+      hasblank=interpolate_1d_blank_write(in, spline, acc);
+
+      /* For a check.
+      {
+        size_t i;
+        double *d;
+        gal_data_t *check=gal_data_copy_to_new_type(in, GAL_TYPE_FLOAT64);
+        d=check->array;
+        for(i=0;i<check->size;++i)
+          printf("%-10zu%f\n", i, d[i]);
+        gal_data_free(check);
+      }
+      */
+
+      /* Set the blank flags, note that `GAL_DATA_FLAG_BLANK_CH' is already set
+         by the top call to `gal_blank_present'. */
+      if(hasblank)
+        in->flag |=  GAL_DATA_FLAG_HASBLANK;
+      else
+        in->flag &= ~GAL_DATA_FLAG_HASBLANK;
+
+      /* Clean up. */
+      gsl_spline_free(spline);
+      gsl_interp_accel_free(acc);
+    }
+}
